package examples.xml;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import examples.beans.Employee;

public class DomParserExample {

	
	List myEmployees;
	Document dom;

 
	public DomParserExample(){
		//egy listat hoz letre, amiben az alkalmazott-objektumokat (employee) fogjuk tarolni
		myEmployees = new ArrayList();
	}

	public void runExample() {
		
		//beolvassa (parse) az  xml allomanyt es letrehozza a dom objektum-hierarchiat
		parseXmlFile();
		
		//sorra bejarja az alkalmazottak (employee) elemeket es letrehoz egy-egy Employee objektumot
		//melyeket egy listaban tarol
		parseDocument();
		
		//Vegigjarja a listat es kiirja az adatokat
		printData();
		
		
	}
	
	
	private void parseXmlFile(){
		// factory lekerese
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		
		try {
			
			//a factory segitsegevel lekerunk egy document builder instanciat
			DocumentBuilder db = dbf.newDocumentBuilder();
			
			//a builder parse metodusat hivjuk meg, ami letrehozza a DOM reprezentaciojat az XML filenak
			dom = db.parse("resource/employees.xml");
			

		}catch(ParserConfigurationException pce) {
			pce.printStackTrace();
		}catch(SAXException se) {
			se.printStackTrace();
		}catch(IOException ioe) {
			ioe.printStackTrace();
		}
	}

	
	private void parseDocument(){
		//lekerjuk a gyoker elemet
		Element docEle = dom.getDocumentElement();
		
		
		//az emloyee elemek csomopont-listajat kerjuk le
		NodeList nl = docEle.getElementsByTagName("Employee");
		if(nl != null && nl.getLength() > 0) {
			for(int i = 0 ; i < nl.getLength();i++) {
				
				//lekerjuk a kovetkezo employee elemet
				Element el = (Element)nl.item(i);
				
				//letrehozzuk az Employee objektumot (lasd lennebb)
				Employee e = getEmployee(el);
				
				//hozzaadjuk a listahoz
				myEmployees.add(e);
			}
		}
	}


	/**
	 * Vesz egy employee elemet, beolvassa a benne tarolt ertekeket, melyekkel
	 * feltolti a letrehozott Employee objektumot
	 */
	private Employee getEmployee(Element empEl) {
		
		//minden <employee> elemre lekerjuk a megfelelo szoveg vagy egesz ertekeket: 
		//name ,id, age, type 
		String name = getTextValue(empEl,"Name");
		int id = getIntValue(empEl,"Id");
		int age = getIntValue(empEl,"Age");

		String type = empEl.getAttribute("type");
		
		//letrehozunk egy uj Employee objektumot az xml csomopontbol kiolvasott adatokkal feltoltve azt
		Employee e = new Employee(name,id,age,type);
		
		return e;
	}


	/**
	 * Vesz egy xml elemet, es egy tag nevet, megkeresi az illeto tag-et es lekeri 
	 * ennek szoveg-tartalmat 
	 * pl. <employee><name>Jancsika</name></employee> eseten, ha az
	 * Element az employee csomopontra mutat a tagName pedig name, a visszaadott ertek Jancsika  
	 */
	private String getTextValue(Element ele, String tagName) {
		String textVal = null;
		NodeList nl = ele.getElementsByTagName(tagName);
		if(nl != null && nl.getLength() > 0) {
			Element el = (Element)nl.item(0);
			textVal = el.getFirstChild().getNodeValue();
		}

		return textVal;
	}

	
	/**
	 * meghivja a getTextValue-t es int erteket terit vissza
	 */
	private int getIntValue(Element ele, String tagName) {
		
		return Integer.parseInt(getTextValue(ele,tagName));
	}
	
	/**
	 * Vegigjarja a listat es a tartalmat a konzolra irja 
	*/
	private void printData(){
		
		System.out.println("No of Employees '" + myEmployees.size() + "'.");
		
		Iterator it = myEmployees.iterator();
		while(it.hasNext()) {
			System.out.println(it.next().toString());
		}
	}

	
	public static void main(String[] args){
		
		DomParserExample dpe = new DomParserExample();
		
		//runExample hivasa:
		dpe.runExample();
	}

}
