package examples.xml;



import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

import examples.beans.Book;


public class XMLCreatorExample {

	
	private List myData;
	private Document dom;

	public XMLCreatorExample() {
		//ures lista letrehozasa az adatok tarolasara
		myData = new ArrayList();

		//lista inicializalasa
		loadData();

		//DOM objektum kinyerese
		createDocument();
	}


	public void runExample(){
		System.out.println("Started .. ");
		createDOMTree();
		printToFile();
		System.out.println("File generated successfully.");
	}

	/**
	 * General egy konyv listat (hardkodolt adatok)
	 * ...a valosagban ezeket az adatokat pl. egy adatbazisbol olvashatnank ki
	 */
	private void loadData(){
		myData.add(new Book("alap Java konyv", "Bruce Eckel","Thinking in Java"));
		myData.add(new Book("Mese", "Fiktiv Iro","Blabla"));
	}

	/**
	 * Letrehozzuk a DOM objektumot
	 */
	private void createDocument() {

		//factory instancia lekerese
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {
		//builder instancia lekerese
		DocumentBuilder db = dbf.newDocumentBuilder();

		// DOM objektum letrehozas
		dom = db.newDocument();

		}catch(ParserConfigurationException pce) {
			
			System.out.println("Error while trying to instantiate DocumentBuilder " + pce);
			System.exit(1);
		}

	}

	/**
	 * Az XML strukturat letrehozo resz
	 */
	private void createDOMTree(){

		//a gyoker elem <Books> letrehozasa
		Element rootEle = dom.createElement("Books");
		dom.appendChild(rootEle);


		Iterator it  = myData.iterator();
		while(it.hasNext()) {
			Book b = (Book)it.next();
			//minden egyes Book objektumra letrehozza a <Book> elemet es hozzafuzi a gyokerhez
			Element bookEle = createBookElement(b);
			rootEle.appendChild(bookEle);
		}

	}

	/**
	 * Seged metodus, mely letrehoz egy <Book> XML elemet
	 * @param b a konyv, amelynek az XML reprezentaciojat kellletrehozni
	 * @return a konyvnek megfelelo XML elem 
	 */
	private Element createBookElement(Book b){

		Element bookEle = dom.createElement("Book");
		bookEle.setAttribute("Subject", b.getSubject());

		//letrehozza az author elemet, illetve author szoveg-csomopontot es hozzacsatolja
		//a bookElement-hez
		Element authEle = dom.createElement("Author");
		Text authText = dom.createTextNode(b.getAuthor());
		authEle.appendChild(authText);
		bookEle.appendChild(authEle);

		//letrehozza a title elemet, illetve title szoveg-csomopontot  es hozzacsatolja
		//a bookElement-hez
		Element titleEle = dom.createElement("Title");
		Text titleText = dom.createTextNode(b.getTitle());
		titleEle.appendChild(titleText);
		bookEle.appendChild(titleEle);

		return bookEle;

	}

	/**
	 * ez a metodus Xerces specifikus osztalyokat hasznal
	 * az XML dokumentum file-ba irasara.
     */
	private void printToFile(){

		try
		{
			//print
			OutputFormat format = new OutputFormat(dom);
			format.setIndenting(true);

			//ha a konzolra szeretnenk irni:
			//XMLSerializer serializer = new XMLSerializer(System.out, format);


			//ha file-ba szeretnenk irni,  fileoutputstream-et hasznalunk system.out helyett
			XMLSerializer serializer = new XMLSerializer(
			new FileOutputStream(new File("book.xml")), format);

			serializer.serialize(dom);

		} catch(IOException ie) {
		    ie.printStackTrace();
		}
	}


	public static void main(String args[]) {

		
		XMLCreatorExample xce = new XMLCreatorExample();

		//runExample futtatasa
		xce.runExample();
	}
}

